    Info << "\nReading earthProperties" << endl;

    IOdictionary earthProperties
    (
        IOobject
        (
            "earthProperties",
            runTime.constant(),
            mesh,
            IOobject::MUST_READ,
            IOobject::NO_WRITE
        )
    );

    dimensionedScalar g(earthProperties.lookup("g"));
    dimensionedVector Omega(earthProperties.lookup("Omega"));

   //const dimensionedScalar filterCoeff(earthProperties.lookup("filterCoeff"));
    //const surfaceScalarField& rdelta=mesh.surfaceInterpolation::deltaCoeffs();
    //const surfaceScalarField filter(filterCoeff*0.25/sqr(rdelta));
    
    Info<< "Reading field h\n" << endl;
    volScalarField h
    (
        IOobject
        (
            "h",
            runTime.timeName(),
            mesh,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        ),
        mesh
    );

    Info<< "Reading field h0 if present\n" << endl;
    volScalarField h0
    (
        IOobject
        (
            "h0",
            "constant",
            mesh,
            IOobject::READ_IF_PRESENT
        ),
        mesh,
        dimensionedScalar("h0", dimLength, 0.0)
    );

    surfaceVectorField SfHat = mesh.Sf()/mesh.magSf();
    
    Info<< "Reading field U\n" << endl;
    volVectorField U
    (
        IOobject
        (
            "U",
            runTime.timeName(),
            mesh,
            IOobject::MUST_READ,
            IOobject::AUTO_WRITE
        ),
        mesh
    );

    Info<< "Reading field hU\n" << endl;
    volVectorField hU
    (
        IOobject
        (
            "hU",
            runTime.timeName(),
            mesh,
            IOobject::READ_IF_PRESENT,
            IOobject::NO_WRITE
        ),
        h*U,
        U.boundaryField().types()
    );

/*    volVectorField Ulatlon
    (
        IOobject
        (
            "Ulatlon",
            runTime.timeName(),
            mesh,
            IOobject::NO_READ,
            IOobject::AUTO_WRITE
        ),
        mesh,
        dimensionedVector("Ulatlon", dimVelocity, vector()),
        zeroGradientFvPatchVectorField::typeName
    );
*/
#   include "createPhi.H"

    volScalarField r = mag(mesh.C());
    volVectorField rHat = mesh.C()/r;

    volVectorField lonHat = vector(0,0,1) ^ rHat;
    lonHat /= mag(lonHat);
    
    surfaceScalarField lonfHat = 
    (
        vector(0,0,1) ^ mesh.Cf()/mag(mesh.Cf())
    )&SfHat;
    
    volVectorField latHat = rHat ^ lonHat;
    latHat /= mag(latHat);

/*    surfaceVectorField lonHatf = vector(0,0,1) ^ rHatf;
    lonHatf /= mag(lonHatf);
    
    surfaceVectorField latHatf = rHatf ^ lonHatf;
    latHatf /= mag(latHatf);
*/
    volVectorField F = ((2.0*Omega) & rHat)*rHat;
    F.rename("F");
    
    // calculate Coriolis on the face centres
    //surfaceVectorField Ff = ((2.0*Omega) & rHatf)*rHatf;
    
    // Coriolis cross product with face area vectors
    //surfaceVectorField SfxFf = mesh.Sf() ^ Ff;

// Courant numbers

surfaceScalarField CoNum = mesh.surfaceInterpolation::deltaCoeffs()*mag(phi)
                         / (fvc::interpolate(h)*mesh.magSf())*runTime.deltaT();

surfaceScalarField waveCoNum = scalar(0.5)
                               *mesh.surfaceInterpolation::deltaCoeffs()
                               *sqrt(mag(fvc::interpolate(h)))
                               *sqrt(g)*runTime.deltaT();
