/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2004 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

Application
    puddleFoam

Description
    Transient solver for shallow-water equations.

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"

volVectorField& constrainVector(volVectorField& V, const volVectorField& rHat)
{
    const volScalarField magV = mag(V);
    V -= rHat*(rHat & V);
    V *= magV/(mag(V) + dimensionedScalar("s", V.dimensions(),VSMALL));
    return V;
}

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

#   include "setRootCase.H"
#   include "createTime.H"
#   include "createMesh.H"
#   include "createFieldsSphere.H"
#   include "createDdtPhiCorrCoeff.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

    Info<< "\nStarting time loop\n" << endl;

    for (runTime++; !runTime.end(); runTime++)
    {
        Info<< "\n Time = " << runTime.timeName() << nl << endl;

#       include "readPISOControls.H"
#       include "CourantNo.H"
//#       include "calcDdtPhiCorrCoeff.H"

        for (int ucorr=0; ucorr<nOuterCorr; ucorr++)
        {
            // Momentum equation
        
            surfaceScalarField phiv("phiv", phi/fvc::interpolate(h));
            volVectorField divCorrect = fvc::div(phiv, hU);
            divCorrect -= constrainVector(divCorrect, rHat);

            fvVectorMatrix hUEqn
            (
                fvm::ddt(hU)
              + fvm::div(phiv, hU)
              - divCorrect
            );

            if (momentumPredictor)
            {
                volVectorField ghgradhh0 = g*h*fvc::grad(h + h0);
                constrainVector(ghgradhh0, rHat);
                {
                    fvVectorMatrix hUEqn2(hUEqn + (F ^ hU) == -ghgradhh0);
                    if (ucorr != nOuterCorr-1)
                    {
                        solve(hUEqn2);
                    }
                    else
                    {
                        hUEqn2.solve(mesh.solver(hU.name() + "Final"));
                    }
                }
                constrainVector(hU, rHat);
                hU.correctBoundaryConditions();
            }
            
            // --- PISO loop

            for (int corr=0; corr<nCorr; corr++)
            {
                surfaceScalarField hf = fvc::interpolate(h);
                volScalarField rUA = 1.0/hUEqn.A();
                surfaceScalarField ghrUAf = g*fvc::interpolate(h*rUA);

                surfaceScalarField phih0 = ghrUAf*mesh.magSf()*fvc::snGrad(h0);

                volVectorField FxhU = F ^ hU;

                hU = rUA*(hUEqn.H() - FxhU);
                constrainVector(hU, rHat);

                phi = (fvc::interpolate(hU) & mesh.Sf()) 
                    + coeff*fvc::ddtPhiCorr(rUA, h, hU, phi)
                    - phih0;

                for (int nonOrth=0; nonOrth<=nNonOrthCorr; nonOrth++)
                {
                    fvScalarMatrix hEqn
                    (
                        fvm::ddt(h)
                      + fvc::div(phi)
                      - fvm::laplacian(ghrUAf, h)
                    );

                    if (ucorr < nOuterCorr-1 || corr < nCorr-1)
                    {
                        hEqn.solve();
                    }
                    else
                    {
                        hEqn.solve(mesh.solver(h.name() + "Final"));
                    }

                    if (nonOrth == nNonOrthCorr)
                    {
                        phi += hEqn.flux();
                    }
                }

                volVectorField gradhh0 = fvc::grad(h + h0);
                constrainVector(gradhh0, rHat);
                hU -= rUA*h*g*gradhh0;
                constrainVector(hU, rHat);
                hU.correctBoundaryConditions();
            }
        }

        U == hU/h;

        runTime.write();

        Info<< "\n    ExecutionTime = "
            << runTime.elapsedCpuTime()
            << " s\n" << endl;
    }

    Info<< "\n end \n";

    return(0);
}


// ************************************************************************* //
